@@ -1,6 +1,6 @@
 ;;; exwm-background.el --- X Background Module for EXWM  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2022-2025 Free Software Foundation, Inc.
 
 ;; Author: Steven Allen <steven@stebalien.com>
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -33,6 +33,7 @@
 ;;; Code:
 
 (require 'exwm-core)
+(eval-when-compile (require 'subr-x)) ;; Needed on 28 for when-let*
 
 (defgroup exwm-background nil
   "Background support."
@@ -45,12 +46,6 @@
   :group 'exwm-background
   (exwm--global-minor-mode-body background))
 
-(defun exwm-background-enable ()
-  "Enable EXWM background support."
-  (message "`exwm-background-enable' is obsolete. Use `exwm-background-mode' instead.")
-  (exwm-background-mode 1))
-(make-obsolete 'exwm-background-enable "Use `exwm-background-mode' instead." "0.30")
-
 (defcustom exwm-background-color nil
   "Background color for Xorg."
   :type '(choice
@@ -207,5 +202,4 @@ may kill this connection when they replace it.")
         exwm-background--atoms nil))
 
 (provide 'exwm-background)
-
 ;;; exwm-background.el ends here
