@@ -151,7 +151,7 @@ See variable `exwm-layout-auto-iconify'."
     (unless (or (exwm-layout--iconic-state-p)
                 (and exwm--floating-frame
                      exwm--desktop
-                     (= 4294967295. exwm--desktop)))
+                     (= #xffffffff exwm--desktop)))
       (exwm--log "Hide #x%x" id)
       (when exwm--floating-frame
         (let* ((container (frame-parameter exwm--floating-frame
