@@ -135,10 +135,6 @@ want to match against EXWM internal variables such as `exwm-title',
                           (cons match config)))
                       value))))
 
-;; FIXME: Make the following values as small as possible.
-(defconst exwm-manage--height-delta-min 5)
-(defconst exwm-manage--width-delta-min 5)
-
 ;; The _MOTIF_WM_HINTS atom (see <Xm/MwmUtil.h> for more details)
 ;; It's currently only used in 'exwm-manage' module
 (defvar exwm-manage--_MOTIF_WM_HINTS nil "_MOTIF_WM_HINTS atom.")
@@ -660,152 +656,140 @@ FRAME is the frame to be deleted."
           (delq (string-to-number (frame-parameter frame 'outer-window-id))
                 exwm-manage--frame-outer-id-list))))
 
+(defun exwm-manage--send-ConfigureNotify (window x y width height)
+  "Send a ConfigureNotify event to WINDOW with X Y WIDTH and HEIGHT."
+  (exwm--log "Reply with ConfigureNotify: %dx%d+%d+%d" width height x y)
+  (xcb:+request exwm--connection
+      (make-instance 'xcb:SendEvent
+                     :propagate 0 :destination window
+                     :event-mask xcb:EventMask:StructureNotify
+                     :event (xcb:marshal
+                             (make-instance
+                              'xcb:ConfigureNotify
+                              :event window :window window
+                              :above-sibling xcb:Window:None
+                              :x x :y y
+                              :width width
+                              :height height
+                              :border-width 0 :override-redirect 0)
+                             exwm--connection))))
+
 (defun exwm-manage--on-ConfigureRequest (data _synthetic)
   "Handle ConfigureRequest event.
 DATA contains unmarshalled ConfigureRequest event data."
   (exwm--log)
-  (let ((obj (make-instance 'xcb:ConfigureRequest))
-        buffer edges width-delta height-delta)
-    (xcb:unmarshal obj data)
-    (with-slots (window x y width height
-                        border-width sibling stack-mode value-mask)
-        obj
-      (exwm--log "#x%x (#x%x) @%dx%d%+d%+d; \
+  (with-slots (window x y width height
+                      border-width sibling stack-mode value-mask)
+      (xcb:unmarshal-new 'xcb:ConfigureRequest data)
+    (exwm--log "#x%x (#x%x) @%dx%d%+d%+d; \
 border-width: %d; sibling: #x%x; stack-mode: %d"
-                 window value-mask width height x y
-                 border-width sibling stack-mode)
-      (if (and (setq buffer (exwm--id->buffer window))
-               (with-current-buffer buffer
-                 (or (exwm-layout--fullscreen-p)
-                     ;; Make sure it's a floating X window wanting to resize
-                     ;; itself.
-                     (or (not exwm--floating-frame)
-                         (progn
-                           (setq edges
-                                 (exwm--window-inside-pixel-edges
-                                  (get-buffer-window buffer t))
-                                 width-delta (- width (- (elt edges 2)
-                                                         (elt edges 0)))
-                                 height-delta (- height (- (elt edges 3)
-                                                           (elt edges 1))))
-                           ;; We cannot do resizing precisely for now.
-                           (and (if (= 0 (logand value-mask
-                                                 xcb:ConfigWindow:Width))
-                                    t
-                                  (< (abs width-delta)
-                                     exwm-manage--width-delta-min))
-                                (if (= 0 (logand value-mask
-                                                 xcb:ConfigWindow:Height))
-                                    t
-                                  (< (abs height-delta)
-                                     exwm-manage--height-delta-min))))))))
-          ;; Send client message for managed windows
-          (with-current-buffer buffer
-            (setq edges
-                  (if (exwm-layout--fullscreen-p)
-                      (with-slots (x y width height)
-                          (exwm-workspace--get-geometry exwm--frame)
-                        (list x y width height))
-                    (exwm--window-inside-absolute-pixel-edges
-                     (get-buffer-window buffer t))))
-            (exwm--log "Reply with ConfigureNotify (edges): %s" edges)
-            (xcb:+request exwm--connection
-                (make-instance 'xcb:SendEvent
-                               :propagate 0 :destination window
-                               :event-mask xcb:EventMask:StructureNotify
-                               :event (xcb:marshal
-                                       (make-instance
-                                        'xcb:ConfigureNotify
-                                        :event window :window window
-                                        :above-sibling xcb:Window:None
-                                        :x (elt edges 0) :y (elt edges 1)
-                                        :width (- (elt edges 2) (elt edges 0))
-                                        :height (- (elt edges 3) (elt edges 1))
-                                        :border-width 0 :override-redirect 0)
-                                       exwm--connection))))
-        (if buffer
-            (with-current-buffer buffer
-              (exwm--log "ConfigureWindow (resize floating X window)")
-              (exwm--set-geometry (frame-parameter exwm--floating-frame
-                                                   'exwm-outer-id)
-                                  nil
-                                  nil
-                                  (+ (frame-outer-width exwm--floating-frame)
-                                     width-delta)
-                                  (+ (frame-outer-height exwm--floating-frame)
-                                     height-delta)))
-          (exwm--log "ConfigureWindow (preserve geometry)")
-          ;; Configure the unmanaged window.
-          ;; But Emacs frames should be excluded.  Generally we don't
-          ;; receive ConfigureRequest events from Emacs frames since we
-          ;; have set OverrideRedirect on them, but this is not true for
-          ;; Lucid build (as of 25.1).
-          (unless (memq window exwm-manage--frame-outer-id-list)
-            (xcb:+request exwm--connection
-                (make-instance 'xcb:ConfigureWindow
-                               :window window
-                               :value-mask value-mask
-                               :x x :y y :width width :height height
-                               :border-width border-width
-                               :sibling sibling
-                               :stack-mode stack-mode)))))))
+               window value-mask width height x y
+               border-width sibling stack-mode)
+    (if-let* ((buffer (exwm--id->buffer window)))
+        (with-current-buffer buffer
+          (if (exwm-layout--fullscreen-p)
+              ;; Fit fullscreen windows to the workspace.
+              (with-slots (x y width height)
+                  (exwm-workspace--get-geometry exwm--frame)
+                (exwm-manage--send-ConfigureNotify
+                 window x y width height))
+            (let* ((edges (exwm--window-inside-absolute-pixel-edges
+                           (get-buffer-window buffer t)))
+                   (window-x (elt edges 0))
+                   (window-y (elt edges 1))
+                   (window-width (- (elt edges 2) window-x))
+                   (window-height (- (elt edges 3) window-y)))
+              (if (not exwm--floating-frame)
+                  ;; If the window isn't floating, fit it to its Emacs window.
+                  (exwm-manage--send-ConfigureNotify
+                   window window-x window-y
+                   window-width window-height)
+                ;; Finally, resize the floating window.
+                (exwm--log "ConfigureWindow (resize floating X window)")
+                (let* ((frame-id (frame-parameter exwm--floating-frame
+                                                  'exwm-outer-id))
+                       (frame-edges (frame-edges exwm--floating-frame
+                                                 'outer-edges))
+                       (frame-width (- (elt frame-edges 2)
+                                       (elt frame-edges 0)))
+                       (frame-height (- (elt frame-edges 3)
+                                        (elt frame-edges 1))))
+                  (exwm--set-geometry
+                   frame-id
+                   nil nil
+                   (unless (= 0 (logand value-mask xcb:ConfigWindow:Width))
+                     (+ frame-width (- width window-width)))
+                   (unless (= 0 (logand value-mask xcb:ConfigWindow:Height))
+                     (+ frame-height (- height window-height)))))))))
+      (exwm--log "ConfigureWindow (preserve geometry)")
+      ;; Configure the unmanaged window.
+      ;; But Emacs frames should be excluded.  Generally we don't
+      ;; receive ConfigureRequest events from Emacs frames since we
+      ;; have set OverrideRedirect on them, but this is not true for
+      ;; Lucid build (as of 25.1).
+      (unless (memq window exwm-manage--frame-outer-id-list)
+        (xcb:+request exwm--connection
+            (make-instance 'xcb:ConfigureWindow
+                           :window window
+                           :value-mask value-mask
+                           :x x :y y :width width :height height
+                           :border-width border-width
+                           :sibling sibling
+                           :stack-mode stack-mode)))))
   (xcb:flush exwm--connection))
 
 (defun exwm-manage--on-MapRequest (data _synthetic)
   "Handle MapRequest event.
 DATA contains unmarshalled MapRequest event data."
-  (let ((obj (make-instance 'xcb:MapRequest)))
-    (xcb:unmarshal obj data)
-    (with-slots (parent window) obj
-      (exwm--log "id=#x%x parent=#x%x" window parent)
-      (if (assoc window exwm--id-buffer-alist)
-          (with-current-buffer (exwm--id->buffer window)
-            (if (exwm-layout--iconic-state-p)
-                ;; State change: iconic => normal.
-                (when (eq exwm--frame exwm-workspace--current)
-                  (pop-to-buffer-same-window (current-buffer)))
-              (exwm--log "#x%x is already managed" window)))
-        (if (/= exwm--root parent)
-            (progn (xcb:+request exwm--connection
-                       (make-instance 'xcb:MapWindow :window window))
-                   (xcb:flush exwm--connection))
-          (exwm--log "#x%x" window)
-          (exwm-manage--manage-window window))))))
+  (with-slots (parent window)
+      (xcb:unmarshal-new 'xcb:MapRequest data)
+    (exwm--log "id=#x%x parent=#x%x" window parent)
+    (if (assoc window exwm--id-buffer-alist)
+        (with-current-buffer (exwm--id->buffer window)
+          (if (exwm-layout--iconic-state-p)
+              ;; State change: iconic => normal.
+              (when (eq exwm--frame exwm-workspace--current)
+                (pop-to-buffer-same-window (current-buffer)))
+            (exwm--log "#x%x is already managed" window)))
+      (if (/= exwm--root parent)
+          (progn (xcb:+request exwm--connection
+                     (make-instance 'xcb:MapWindow :window window))
+                 (xcb:flush exwm--connection))
+        (exwm--log "#x%x" window)
+        (exwm-manage--manage-window window)))))
 
 (defun exwm-manage--on-UnmapNotify (data _synthetic)
   "Handle UnmapNotify event.
 DATA contains unmarshalled UnmapNotify event data."
-  (let ((obj (make-instance 'xcb:UnmapNotify)))
-    (xcb:unmarshal obj data)
-    (with-slots (window) obj
-      (exwm--log "id=#x%x" window)
-      (exwm-manage--unmanage-window window t))))
+  (with-slots (window)
+      (xcb:unmarshal-new 'xcb:UnmapNotify data)
+    (exwm--log "id=#x%x" window)
+    (exwm-manage--unmanage-window window t)))
 
 (defun exwm-manage--on-MapNotify (data _synthetic)
   "Handle MapNotify event.
 DATA contains unmarshalled MapNotify event data."
-  (let ((obj (make-instance 'xcb:MapNotify)))
-    (xcb:unmarshal obj data)
-    (with-slots (window) obj
-      (when (assoc window exwm--id-buffer-alist)
-        (exwm--log "id=#x%x" window)
-        ;; With this we ensure that a "window hierarchy change" happens after
-        ;; mapping the window, as some servers (XQuartz) do not generate it.
-        (with-current-buffer (exwm--id->buffer window)
-          (if exwm--floating-frame
-              (xcb:+request exwm--connection
-                  (make-instance 'xcb:ConfigureWindow
-                                 :window window
-                                 :value-mask xcb:ConfigWindow:StackMode
-                                 :stack-mode xcb:StackMode:Above))
+  (with-slots (window)
+      (xcb:unmarshal-new 'xcb:MapNotify data)
+    (when (assoc window exwm--id-buffer-alist)
+      (exwm--log "id=#x%x" window)
+      ;; With this we ensure that a "window hierarchy change" happens after
+      ;; mapping the window, as some servers (XQuartz) do not generate it.
+      (with-current-buffer (exwm--id->buffer window)
+        (if exwm--floating-frame
             (xcb:+request exwm--connection
                 (make-instance 'xcb:ConfigureWindow
                                :window window
-                               :value-mask (logior xcb:ConfigWindow:Sibling
-                                                   xcb:ConfigWindow:StackMode)
-                               :sibling exwm--guide-window
-                               :stack-mode xcb:StackMode:Above))))
-        (xcb:flush exwm--connection)))))
+                               :value-mask xcb:ConfigWindow:StackMode
+                               :stack-mode xcb:StackMode:Above))
+          (xcb:+request exwm--connection
+              (make-instance 'xcb:ConfigureWindow
+                             :window window
+                             :value-mask (logior xcb:ConfigWindow:Sibling
+                                                 xcb:ConfigWindow:StackMode)
+                             :sibling exwm--guide-window
+                             :stack-mode xcb:StackMode:Above))))
+      (xcb:flush exwm--connection))))
 
 (defun exwm-manage--on-DestroyNotify (data synthetic)
   "Handle DestroyNotify event.
@@ -813,10 +797,9 @@ DATA contains unmarshalled DestroyNotify event data.
 SYNTHETIC indicates whether the event is a synthetic event."
   (unless synthetic
     (exwm--log)
-    (let ((obj (make-instance 'xcb:DestroyNotify)))
-      (xcb:unmarshal obj data)
-      (exwm--log "#x%x" (slot-value obj 'window))
-      (exwm-manage--unmanage-window (slot-value obj 'window)))))
+    (with-slots (window) (xcb:unmarshal-new 'xcb:DestroyNotify data)
+      (exwm--log "#x%x" window)
+      (exwm-manage--unmanage-window window))))
 
 (defun exwm-manage--init ()
   "Initialize manage module."
