@@ -797,17 +797,20 @@ INDEX must not exceed the current number of workspaces."
   "Delete the workspace FRAME-OR-INDEX."
   (interactive)
   (exwm--log "%s" frame-or-index)
-  (when (< 1 (exwm-workspace--count))
-    (let ((frame (if frame-or-index
-                     (exwm-workspace--workspace-from-frame-or-index
-                      frame-or-index)
-                   exwm-workspace--current)))
-      ;; Transfer over any surrogate minibuffers before trying to delete the workspace.
-      (let ((minibuf (minibuffer-window frame))
-            (newminibuf (minibuffer-window (exwm-workspace--get-next-workspace frame))))
-        (dolist (f (filtered-frame-list (lambda (f) (eq (frame-parameter f 'minibuffer) minibuf))))
-          (set-frame-parameter f 'minibuffer newminibuf)))
-      (delete-frame frame))))
+  (if-let* ((frame (if frame-or-index
+                       (exwm-workspace--workspace-from-frame-or-index
+                        frame-or-index)
+                     exwm-workspace--current))
+            (next-frame (exwm-workspace--get-next-workspace
+                         frame (not (exwm-workspace--active-p frame)))))
+      (progn
+        ;; Transfer over any surrogate minibuffers before trying to delete the workspace.
+        (let ((minibuf (minibuffer-window frame))
+              (newminibuf (minibuffer-window next-frame)))
+          (dolist (f (filtered-frame-list (lambda (f) (eq (frame-parameter f 'minibuffer) minibuf))))
+            (set-frame-parameter f 'minibuffer newminibuf)))
+        (delete-frame frame))
+    (error "All other workspaces are active")))
 
 (defun exwm-workspace--set-desktop (id)
   "Set _NET_WM_DESKTOP for X window ID."
@@ -1130,33 +1133,31 @@ account when calculating the height."
 (defun exwm-workspace--on-ConfigureNotify (data _synthetic)
   "Adjust the container to fit the minibuffer frame.
 DATA contains unmarshalled ConfigureNotify event data."
-  (let ((obj (make-instance 'xcb:ConfigureNotify)) y)
-    (xcb:unmarshal obj data)
-    (with-slots (window height) obj
-      (when (eq (frame-parameter exwm-workspace--minibuffer 'exwm-outer-id)
-                window)
-        (exwm--log)
-        (when (and (floatp max-mini-window-height)
-                   (> height (* max-mini-window-height
-                                (exwm-workspace--current-height))))
-          (setq height (floor
-                        (* max-mini-window-height
-                           (exwm-workspace--current-height))))
-          (xcb:+request exwm--connection
-              (make-instance 'xcb:ConfigureWindow
-                             :window window
-                             :value-mask xcb:ConfigWindow:Height
-                             :height height)))
-        (when (/= (exwm-workspace--count) (length exwm-workspace--workareas))
-          ;; There is a chance the workareas are not updated timely.
-          (exwm-workspace--update-workareas))
-        (with-slots ((y* y) (height* height))
-            (exwm-workspace--workarea exwm-workspace-current-index)
-          (setq y (if (eq exwm-workspace-minibuffer-position 'top)
-                      (- y*
-                         exwm-workspace--attached-minibuffer-height)
-                    (+ y* height* (- height)
-                       exwm-workspace--attached-minibuffer-height))))
+  (with-slots (window height)
+      (xcb:unmarshal-new 'xcb:ConfigureNotify data)
+    (when (eq (frame-parameter exwm-workspace--minibuffer 'exwm-outer-id)
+              window)
+      (exwm--log)
+      (when (and (floatp max-mini-window-height)
+                 (> height (* max-mini-window-height
+                              (exwm-workspace--current-height))))
+        (setq height (floor
+                      (* max-mini-window-height
+                         (exwm-workspace--current-height))))
+        (xcb:+request exwm--connection
+            (make-instance 'xcb:ConfigureWindow
+                           :window window
+                           :value-mask xcb:ConfigWindow:Height
+                           :height height)))
+      (when (/= (exwm-workspace--count) (length exwm-workspace--workareas))
+        ;; There is a chance the workareas are not updated timely.
+        (exwm-workspace--update-workareas))
+      (let ((y (with-slots ((y* y) (height* height))
+                   (exwm-workspace--workarea exwm-workspace-current-index)
+                 (if (eq exwm-workspace-minibuffer-position 'top)
+                     (- y* exwm-workspace--attached-minibuffer-height)
+                   (+ y* height* (- height)
+                      exwm-workspace--attached-minibuffer-height)))))
         (xcb:+request exwm--connection
             (make-instance 'xcb:ConfigureWindow
                            :window (frame-parameter exwm-workspace--minibuffer
@@ -1375,14 +1376,25 @@ ALIST is an action alist, as accepted by function `display-buffer'."
                frame exwm-workspace-current-index original-index))
     (run-hooks 'exwm-workspace-list-change-hook)))
 
-(defun exwm-workspace--get-next-workspace (frame)
+(defun exwm-workspace--get-next-workspace (frame &optional allow-active)
   "Return the next workspace if workspace FRAME were removed.
-Return nil if FRAME is the only workspace."
+Return nil there are no other worksapces or ALLOW-ACTIVE is non-nil and
+all other workspaces are currently visible on other monitors."
   (let* ((index (exwm-workspace--position frame))
-         (lastp (= index (1- (exwm-workspace--count))))
-         (nextw (elt exwm-workspace--list (+ index (if lastp -1 +1)))))
-    (unless (eq frame nextw)
-      nextw)))
+         (count (exwm-workspace--count)))
+    (or
+     (cl-loop for i from (1+ index) below count
+              for nextw = (elt exwm-workspace--list i)
+              when (and (not (eq frame nextw))
+                        (or allow-active
+                            (exwm-workspace--active-p nextw)))
+              return nextw)
+     (cl-loop for i from (1- index) downto 0
+              for nextw = (elt exwm-workspace--list i)
+              when (and (not (eq frame nextw))
+                        (or allow-active
+                            (exwm-workspace--active-p nextw)))
+              return nextw))))
 
 (defun exwm-workspace--remove-frame-as-workspace (frame &optional quit)
   "Stop treating FRAME as a workspace.
@@ -1411,8 +1423,11 @@ When QUIT is non-nil cleanup avoid communicating with the X server."
           (when (memq other-frame following-frames)
             (exwm-workspace--set-desktop (car pair)))))
       ;; If the current workspace is deleted, switch to next one.
-      (when (eq frame exwm-workspace--current)
-        (exwm-workspace-switch next-frame))))
+      (if (eq frame exwm-workspace--current)
+          (exwm-workspace-switch next-frame)
+        ;; Otherwise, update the workspace index.
+        (setq exwm-workspace-current-index
+              (exwm-workspace--position next-frame)))))
   ;; Reparent out the frame.
   (let ((outer-id (frame-parameter frame 'exwm-outer-id)))
     (xcb:+request exwm--connection
